package stericson.busybox.donate.activities;

import java.lang.reflect.Method;
import java.util.Random;

import stericson.busybox.donate.R;
import stericson.busybox.donate.interfaces.ChoiceCallback;
import stericson.busybox.donate.jobs.AsyncJob;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnCancelListener;
import android.content.res.Configuration;
import android.graphics.Typeface;
import android.os.Bundle;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.PopupWindow;
import android.widget.TextView;
import android.widget.Toast;

import com.stericson.RootTools.RootTools;

public class BaseActivity extends Activity {

    public PopupWindow pw;
    public boolean endApplication;

    public Typeface tf;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        RootTools.debugMode = true;
        try {
            tf = Typeface.createFromAsset(getAssets(), "fonts/DJGROSS.ttf");
        } catch (Exception e) {}
    }

    @Override
    protected void onDestroy() {
        AsyncJob.cancelAllJobs(this);
        super.onDestroy();
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        // We do nothing here. We're only handling this to keep orientation
        // or keyboard hiding from causing the activity to restart.
    }

    public void initiatePopupWindow(String text, boolean endApplication,
                                    Activity context) {

        if (pw != null && pw.isShowing())
            pw.dismiss();

        if (!context.isFinishing()) {
            this.endApplication = endApplication;

            LayoutInflater inflater = (LayoutInflater) context
                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            // Inflate the view from a predefined XML layout
            View layout = inflater.inflate(R.layout.popupwindow, null);
            pw = new PopupWindow(layout, LayoutParams.MATCH_PARENT,
                    LayoutParams.MATCH_PARENT);

            context.findViewById(R.id.pop).post(new Runnable() {
                public void run() {
                    pw.showAtLocation(findViewById(R.id.pop), Gravity.CENTER, 0, 0);
                }
            });

            TextView header = (TextView) layout.findViewById(R.id.header_main);
            header.setTypeface(tf);

            TextView textView = (TextView) layout.findViewById(R.id.content);
            textView.setText(text);
        }
    }

    public void makeChoice(final ChoiceCallback choice, final int id, int title, int content, int positive, int negative) {
        new AlertDialog.Builder(this)
                .setTitle(title)
                .setMessage(content)
                .setPositiveButton(positive, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int whichButton) {
                        choice.choiceMade(true, id);
                    }
                }).setNegativeButton(negative, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                choice.choiceMade(false, id);
            }
        }).setOnCancelListener(new OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialog) {
                choice.choiceCancelled(id);
            }
        }).show();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK && event.getRepeatCount() == 0) {

            if (pw != null && pw.isShowing()) {
                close(new View(this));
            } else {
                finish();
                randomAnimation();
            }

            return true;
        }

        return super.onKeyDown(keyCode, event);
    }

    public void randomAnimation() {
        Random random = new Random();
        switch (random.nextInt(3)) {
            case 0:
                overridePendingTransition(this, R.anim.enter_scalein,
                        R.anim.exit_slideout);
                break;
            case 1:
                overridePendingTransition(this, R.anim.enter_dropin,
                        R.anim.exit_dropout);
                break;
            case 2:
                overridePendingTransition(this, R.anim.enter_slidein,
                        R.anim.exit_slideout);
                break;
        }
    }

    public void close(View v) {
        pw.dismiss();
        if (endApplication) {
            finish();
            randomAnimation();
        }
    }

    private static Method overridePendingTransition;

    static {
        try {
            overridePendingTransition = Activity.class
                    .getMethod(
                            "overridePendingTransition", new Class[]{Integer.TYPE, Integer.TYPE}); //$NON-NLS-1$
        } catch (NoSuchMethodException e) {
            overridePendingTransition = null;
        }
    }

    /**
     * Calls Activity.overridePendingTransition if the method is available
     * (>=Android 2.0)
     *
     * @param activity  the activity that launches another activity
     * @param animEnter the entering animation
     * @param animExit  the exiting animation
     */
    public static void overridePendingTransition(Activity activity,
                                                 int animEnter, int animExit) {
        if (overridePendingTransition != null) {
            try {
                overridePendingTransition.invoke(activity, animEnter, animExit);
            } catch (Exception e) {
                // do nothing
            }
        }
    }

    public void debug(View v) {
        RootTools.debugMode = !RootTools.debugMode;
        Toast.makeText(this, "Debug is " + RootTools.debugMode, Toast.LENGTH_LONG).show();
    }
}
